<?php
/**************************************************
  Coppermine 1.5.x Plugin - gallery_merge
  Plugin Support for minicms (MiniCMS)
**************************************************/

/*
   page included into merge_db.php to handle simulate and execution of merges for a specific plugin
   page should be named 'merge_plugin_{pluginfoldername}.php and placed in the gallery_merge/data directory
*/

// Make sure that this file can't be accessed directly, but only from within the Coppermine user interface
if (!defined('IN_COPPERMINE')) {
    die('Not in Coppermine...');
}

if (!GALLERY_ADMIN_MODE) {
    cpg_die(ERROR, $lang_errors['access_denied'], __FILE__, __LINE__);
}

// ********************
// PLUGIN Merge Support
// ********************
/*
  'MiniCMS' is a minimalistic content management system for Coppermine.
  Creates 2 tables - cms_config and cms.
  - cms_config is handled like the main cpg config table - comparing entries and reporting differences only.
  - cms table contains the actual content - and is merged to the TARGET system.
    Special handling needed for:
    * `conid` field can refer to a category, album, or picture based on value in `type` field (conditional mapping)
    * cpos field needs mapping/updates not handled by normal gallery merge functions - custom functions included here.
*/

/*
  See readme_plugin_doc.txt for detailed comments and parameters
  Unused variables or functions sections can be deleted.
*/

// ***********************
// Define Plugin Variables
// ***********************
// if any variables are required with GLOBAL scope, add them here - named $plugin_gallery_merge_{plugin_folder}_{name}
//   used to preserve/pass data between pre_merge and post_merge functions

$plugin_gallery_merge_minicms_cpos_map = '';                                    // data to be passed from pre_merge to post_merge functions

// ***********************
// Define Plugin Functions
// ***********************
// if any functions are required, add them here - named gallery_merge_plugin_{plugin_folder}_{name}
// to call function around processing for a given table (process_type = 'process_table') or instead of normal functions (process_type = 'custom'),
//   specify function name in 'pre_merge_function' field of plugin_table_array below to call BEFORE normal processing.
//   specify function name in 'post_merge_function' field of plugin_table_array below to call AFTER normal processing.
//   (for custom, either or both can be specified... no processingwill occur between them.)
// replace values in {} with applicable values
// to pass data between 'pre_merge' and 'post_merge' functions, specify global variables above and reference with global statement in function
// to save data across restarts, place data in array, and use sample checkpoint logic..
//   (typically should only be necessary in 'pre_merge' in event merge or 'post_merge' are interrupted - and process cannot just be re-run)


function gallery_merge_plugin_minicms_map_cpos() {
  global $CONFIG, $SOURCE, $message_level, $debug_level, $debug_trace;          // core variables and settings
  global $restart_flag;                                                         // checkpoint restart setting
  global $plugin_gallery_merge_execute;                                         // execution flags - only perform UPDATES if set to TRUE
  global $source_mapping_cms, $source_mapping_cms_key;                          // access to cms mapping table and key
  global $plugin_gallery_merge_minicms_cpos_map;                                // access to global variables defined above

  // minicms_map_cpos called as pre_merge_function to build unique mapping table for the cms table.
  // minicms uses a type field to indicate the level the comment applies to, a conid to show the specific item at that level, and cpos to hold position within that item...
  //  ie: type 2 is an entry for a picture - conid would be pid, and cpos would be the position of that enty if more than one.
  // build a mapping array from target that holds max cpos of each conid within a type - not supported by the standard gallery_merge mapping
  // (we do use the standard mapping as well to set all but the cpos field)

  // checkpoint restart calls implemented in case main merge is interrupted - rebuilding map after some inserts can produce undesired results.
  // map array saved in checkpoint at end of function, and reloaded on restart if available.
  // common initialization
  $func_name = substr(__FUNCTION__, 21);                                        // strip off gallery_merge_plugin_
  if ($debug_level >= 3) { $debug_trace[] = "Plugin Function $func_name Entered"; }  // debug footprint
  $process_name = "PROCESS_PLUGIN_$func_name";                                  // build processs name based on this function

  // if checkpointing data we need back on restart...
  if ($restart_flag == 'RESTART') {
    // this is a restart - see if we have data...
    $plugin_gallery_merge_minicms_cpos_map = gallery_merge_checkpoint_load($process_name);  // request our checkpoint data
  }
  // if returned data is array - we have our data back (if only one chackpoint taken at end - should mean we are done processing)
  // if returned data == TRUE - checkpoint existed, but no data included
  // if returned data == FALSE - no checkpoint found, process as never run
  if (is_array($plugin_gallery_merge_minicms_cpos_map)) {
    // data loaded from checkpoint... we are done
    return;
  }

  // not a restart - or checkpoint not available
  $plugin_gallery_merge_minicms_cpos_map = array();
  // get data from TARGET table
  $sql = "SELECT conid, cpos, type FROM {$CONFIG['TABLE_CMS']} ORDER BY type, conid, cpos DESC";
  $result = cpg_db_query($sql);                                                 // execute against TARGET DB
  while ($target_row = cpg_db_fetch_row($result))  {
    if (!array_key_exists($target_row['conid'], $plugin_gallery_merge_minicms_cpos_map[$target_row['type']])) {
      // no key yet... save highest cpos for conid within type (SQL sort gives us largest value first)
      $plugin_gallery_merge_minicms_cpos_map[$target_row['type']][$target_row['conid']] = $target_row['cpos'];
    }
  }

  // if all ok and data to save for restart... in array $plugin_gallery_merge_minicms_cpos_map
  gallery_merge_checkpoint('db', 'checkpoint', '', "$process_name:COMPLETE", $plugin_gallery_merge_minicms_cpos_map);
  if ($debug_level >= 3) { $debug_trace[] = "Plugin Function $func_name Completed"; }  // debug footprint
  cpg_db_free_result($result);
} // end function


function gallery_merge_plugin_minicms_update_cpos() {
  global $CONFIG, $SOURCE, $message_level, $debug_level, $debug_trace;
  global $plugin_gallery_merge_execute;                                         // execution flags - only perform UPDATES if set to TRUE
  global $source_mapping_cms, $source_mapping_cms_key;                          // access to cms mapping table and key
  global $source_mapping_categories, $source_mapping_categories_key;            // access to categories mapping table and key
  global $source_mapping_albums, $source_mapping_albums_key;                    // access to albums mapping table and key
  global $source_mapping_pictures, $source_mapping_pictures_key;                // access to pictures mapping table and key
  global $plugin_gallery_merge_minicms_cpos_map;                                // access to our pre_merge functions data

  // minicms_update_cpos called as post_merge_function to update/correct cpos entries for the new additions to the cms table
  // use the standard map we requested processing cms table, and map create in pre-merge function to see if we inserted any rows that reference an already existing conid in target.
  // for any matches, update the cpos to correct value. new source entries will follow already existing target entries.
  // no checkpoint restart needed here - as we can always rerun this portion.

  if ($debug_level >= 3) { $debug_trace[] = "Plugin Function minicms_update_cpos Entered"; }

  // New records have been inserted, but available mapping wouldn't handle cpos updates at insert time
  // traverse mapping table. Since this table processed INSERTing all rows found - every row in mapping table represents an insert - calculate new value for cpos
  // (only time we wouldn't have just inserted something in mapping array would be on a restart following failure processing the cms table - but in that case we want to update anyway...
  //  very unlikely to have an exact match between source and target on cms entry - we match against full description and text to see if we need to insert.)
  foreach ($source_mapping_cms AS $row_key => $row_data) {
    if (array_key_exists($row_data['conid'], $plugin_gallery_merge_minicms_cpos_map[$row_data['type']])) {
      // cpos_map contains an entry for this type and conid... calculate new cpos
      $cpos = $row_data['cpos'] + $plugin_gallery_merge_minicms_cpos_map[$row_data['type']][$row_data['conid']];
      // update translate table
      $source_mapping_cms[$row_key]['target_cpos'] = $cpos;
      // update database
      $update_sql = "UPDATE {$CONFIG['TABLE_CMS']} SET cpos = '$cpos' WHERE ID = '{$row_data['target_ID']}'";  // set sql
      gallery_merge_debug_sql($update_sql);
      if ($plugin_gallery_merge_execute) {
        // execute mode!
        cpg_db_query($update_sql);                                              // execute against TARGET DB if execute mode only
      }
    }
  }
  if ($debug_level >= 3) { $debug_trace[] = "Plugin Function minicms_update_cpos Completed"; }
} // end function


// ***********************
// Define Plugin Variables
// ***********************
//** Update variables to support merge of this plugin's data tables
//** see readme_plugin_doc.txt for detailed information on parameters
$plugin_name = 'MiniCMS';                                                 // name as it appears in cpg_plugins table and data/plugin_data.php
// array of plugin's tables
$plugin_table_array = array(
                            'cms_config' => array(                                                            // table name that follows the cpg table prefix
                                        'process_type' => 'compare_values',                                   // see detailed options in readme_plugin_doc.txt
                                                       // ****** following fields only required if 'process_type' = 'compare_values' ******
                                 'compare_name_column' => 'name',                                             // name of column containing the variable names
                                'compare_value_column' => 'value'                                            // name of the column containing the variable setting
                                                   ),  // end table definition

                            'cms' => array(                                                                   // table name that follows the cpg table prefix
                                        'process_type' => 'process_table',                                    // see detailed options in readme_plugin_doc.txt
                                                       // ****** following fields only required if 'process_type' = 'process_table' or 'custom' ******
                                           'table_key' => 'ID',                                               // key field name
                                             'orderby' => 'ORDER BY type, conid, cpos',                       // order by key field is default to assist restart - override here if needed
                                      'compare_fields' => 'type,conid,title,content',                         // field(s) - comma separated - to match source to target (identify duplicates/inserts)
                                        'compare_type' => 'UNIQUE',                                           // type of compare: UNIQUE, IGNORE, MERGE
                                           'skip_keys' => array('ID', 'modified'),                            // set keys/columns to bypass for insert (typically auto-increment fields)
                                       'obsolete_keys' => array(''),                                          // set keys/columns that are now obsolete - but might still exist in older installs
                                           'zero_keys' => array(''),                                          // set keys/columns to 0 value on insert (typically data not yet available... to be updated later)
                                          'map_values' => array('/title', '/type', 'conid', 'cpos'),          // set mapping values to set fields to build array for later processing
                                'skip_keys_difference' => array(),                                            // set keys to bypass for comparing like rows in source and target (like passwords...)
                                        'translations' => array('conid.0' => 'categories.conid.cid.type=0',
                                                                'conid.1' => 'albums.conid.aid.type=1',
                                                                'conid.2' => 'pictures.conid.pid.type=2'),    // set translations to be done... fieldname => table.keyvalue.field_to_use.optional_condition
                                                                // type=3 is section - which doesn't use conid - no need to translate
                                                       // ****** following fields (at least one) only required if 'process_type' = 'custom' - Optional for 'process_type = 'process_table' ******
                                  'pre_merge_function' => 'gallery_merge_plugin_minicms_map_cpos',            // function to call before execution of merge script... (process types of 'process_table' and 'custom' only)
                                 'post_merge_function' => 'gallery_merge_plugin_minicms_update_cpos'          // function to call following execution of merge script... (process types of 'process_table' and 'custom' only)
                                                   ),  // end table definition

                           ); // end all tables definitions

//** add this script name to the merge_script variable for this plugin in data/plugin_data.php to have it invoked when plugin is in use in source and target galleries.
?>